package ui.controller;

import java.util.ArrayList;

import Interfaces.CategoryListener;
import Interfaces.ObjectListener;
import classes.CpsEdge;
import classes.CpsObject;
import ui.model.Model;

public class CanvasController {

	private Model MODEL;
	private SearchController sC;

	public CanvasController(Model model, SearchController sC) {
		this.MODEL = model;
		this.sC = sC;
	}

	/**
	 * Add an CpsObject to the model and notify the ObjectListener for update.
	 * 
	 * @param object
	 *            CpsObject to be added.
	 */
	public void addObject(CpsObject object) {
		// String objName = object.getObjName();
		// object.setSav("Canvas");
		// object.setConnections(new ArrayList<>());
		int i = 0;
		while (sC.searchHolonObject(object.getName(), MODEL.getObjectsOnCanvas()) != null) {
			if (object.getObjName().contains("_"))
				object.setObjName(object.getObjName().substring(0, object.getObjName().indexOf('_')));
			String name = object.getObjName() + "_" + i;
			object.setObjName(name);
			object.setName(name);
			i++;
		}

		MODEL.getObjectsOnCanvas().add(object);
		notifyObjListeners();
		// System.out.println("Added: " + objName);
	}

	public void addNewObject(CpsObject object) {
		object.setSav("Canvas");
		object.setConnections(new ArrayList<>());
		addObject(object);
	}

	public void addObjectListener(ObjectListener objLis) {
		MODEL.getObjectListeners().add(objLis);
	}

	/**
	 * notifies all listeners about changes in the Canvas
	 */
	public void notifyObjListeners() {
		for (ObjectListener l : MODEL.getObjectListeners()) {
			l.onChange(MODEL.getObjectsOnCanvas());
		}
	}

	/**
	 * Deletes an CpsObject on the Canvas and its connections
	 * 
	 * @param CpsObject
	 */
	public void deleteObjectOnCanvas(CpsObject obj) {
		CpsEdge e = null;
		for (CpsObject cps : MODEL.getObjectsOnCanvas()) {
			for (CpsEdge p : cps.getConnections()) {
				if (p.getA() == obj || p.getB() == obj) {
					e = p;
				}
			}
			cps.getConnectedTo().remove(e);
			MODEL.getEdgesOnCanvas().remove(e);
		}
		MODEL.getObjectsOnCanvas().remove(obj);
	}

	/**
	 * @param objectsOnCanvas
	 *            the objectsOnCanvas to set
	 */
	public void addEdgeOnCanvas(CpsEdge edge) {
		MODEL.getEdgesOnCanvas().add(edge);
	}

	/**
	 * @param edgesOnCanvas
	 *            the edge to remove
	 */
	public void removeEdgesOnCanvas(CpsEdge edge) {
		MODEL.getEdgesOnCanvas().remove(edge);
	}

}
